@@ -3,22 +3,23 @@
  *   Code is stol^H^H^H^Hbased on wmppp, wmload, and wmtme  
  *
  *   Author: Ben Cohen (buddog@aztec.asu.edu)
- *
+ *   Contributors:
+ *       Guilhem valentin <valentin@via.ecp.fr>
+ *       Chris Conn <cconn@abacom.com>
+ *       Kevin Pulo <kev@zip.com.au>
+ *       
  *   This program is distributed under the GPL license --
- *   !! Except for those parts ( face images ) which are certaintly 
+ *   !! Except for those parts ( doom face images ) which are certaintly 
  *   owned by ID Software. !!
  *
  ***********************************************************************/
 
 
-    
-
-
-#include <Xlib.h>
-#include <Xutil.h>
-#include <xpm.h>
-#include <extensions/shape.h>
-#include <keysym.h>
+#include <X11/Xlib.h>
+#include <X11/Xutil.h>
+#include <X11/xpm.h>
+#include <X11/extensions/shape.h>
+#include <X11/keysym.h>
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
@@ -26,35 +27,88 @@
 #include <unistd.h>
 
 
-/* XPM files */
+/* Common XPM Images */
 
-#include "backdrop.xpm"
-/* #include "buttons.xpm" */
-#include "face_0c.xpm"
-#include "face_0r.xpm"
-#include "face_0l.xpm"
-#include "face_1c.xpm"
-#include "face_1r.xpm"
-#include "face_1l.xpm"
-#include "face_2c.xpm"
-#include "face_2r.xpm"
-#include "face_2l.xpm"
-#include "face_3c.xpm"
-#include "face_3r.xpm"
-#include "face_3l.xpm"
-#include "face_4c.xpm"
-#include "face_4r.xpm"
-#include "face_4l.xpm"
-#include "face_god.xpm"
+#include "common/backdrop.xpm"
+#include "common/mask.xbm"
+
+
+/* Doom XPM Images */
+
+#include "doom/doom_0c.xpm"
+#include "doom/doom_0r.xpm"
+#include "doom/doom_0l.xpm"
+#include "doom/doom_1c.xpm"
+#include "doom/doom_1r.xpm"
+#include "doom/doom_1l.xpm"
+#include "doom/doom_2c.xpm"
+#include "doom/doom_2r.xpm"
+#include "doom/doom_2l.xpm"
+#include "doom/doom_3c.xpm"
+#include "doom/doom_3r.xpm"
+#include "doom/doom_3l.xpm"
+#include "doom/doom_4c.xpm"
+#include "doom/doom_4r.xpm"
+#include "doom/doom_4l.xpm"
+#include "doom/doom_god.xpm"
+
+
+/* Penguin XPM Images: ( Thanks to  Chris Conn <cconn@abacom.com> ) */
+
+#include "penguin/penguin_0c.xpm"
+#include "penguin/penguin_0r.xpm"
+#include "penguin/penguin_0l.xpm"
+#include "penguin/penguin_1c.xpm"
+#include "penguin/penguin_1r.xpm"
+#include "penguin/penguin_1l.xpm"
+#include "penguin/penguin_2c.xpm"
+#include "penguin/penguin_2r.xpm"
+#include "penguin/penguin_2l.xpm"
+#include "penguin/penguin_3c.xpm"
+#include "penguin/penguin_3r.xpm"
+#include "penguin/penguin_3l.xpm"
+#include "penguin/penguin_4c.xpm"
+#include "penguin/penguin_4r.xpm"
+#include "penguin/penguin_4l.xpm"
+#include "penguin/penguin_god.xpm"
+
+
+char ** face_name[] = { 
+        doom_0l, doom_0c, doom_0r,
+        doom_1l, doom_1c, doom_1r,
+        doom_2l, doom_2c, doom_2r,
+        doom_3l, doom_3c, doom_3r, 
+        doom_4l, doom_4c, doom_4r, doom_god,
+        penguin_0l, penguin_0c, penguin_0r,
+        penguin_1l, penguin_1c, penguin_1r,
+        penguin_2l, penguin_2c, penguin_2r,
+        penguin_3l, penguin_3c, penguin_3r, 
+        penguin_4l, penguin_4c, penguin_4r, penguin_god 
+             };
+
+/* NOTE: doom_god and penguin_god are currently never used.
+         If you have any good ideas for what to use them for,
+         let me know. */
+
+
+
+/* The face images are not necessairly the same dimentions. 
+   Specifty the x and y offset into window for each face set.
+   set 0=doom, set 1=penguin 
+                      set 0   set 1
+                      X  Y    X  Y                          */
+int face_offset[] = { 6, 5,   3, 5 };
 
-#include "mask.xbm"
 
 /***********************************************************************
  *         Globals..   
  ***********************************************************************/
 
-#define VERSION "wmdl 1.3.7"
+#define VERSION "wmdl 1.4.1"
 #define LEVELS 5
+#define FACE_SETS 2
+#define IMAGES_PER_FACE_SET 16
+#define TOTAL_FACES 32    /* equals FACE_SETS * IMAGES_PER_FACE_SET */
 
 typedef enum { YES, NO } yesno_type;
 typedef enum { CPU, LOAD, COMMAND } mode_type;
@@ -84,16 +138,14 @@ Pixel bg_pixel, fg_pixel;
 Pixmap pixmask;
 
 XpmIcon_type template, visible;
-XpmIcon_type face[16];
-
-int Verbose = 0;
+XpmIcon_type face[TOTAL_FACES];  /* array of images */
 
 
 /***********************************************************************
  *         Function Prototypes
  ***********************************************************************/
 
-void redraw( int level, int dirrection );
+void redraw( int face_set, int level, int dirrection );
 void getPixmaps();
 int whichButton( int x, int y );
 int flush_expose(Window w);
@@ -105,6 +157,14 @@ void new_dirrection( int *dirrection );
 void refresh( command_type command, float scale_factor, int dirrection );
 
 
+int Verbose = 0;
+int face_set = 0; /* default inital face set. */
+
+
+
+/* The face images are not necessairly the same dimentions. 
+   Specifty the x and y offset into window for each face set.
+   0=doom, 1=penguin */
 
 
 
@@ -159,10 +219,10 @@ int main( int argc, char ** argv )
     for ( i=1; i<argc; i++ ) {
         if ( *argv[i] == '-' ) {
             switch ( *(argv[i]+1) ) {
-                case 'v':
+                case 'v': /* Set Verbose Mode */
                     Verbose = 1;
                     break;
-                case 's':
+                case 's': /* Use X11 Shaped Windows or Not */
                     if ( ++i >= argc ) show_usage();
                     strcpy( is_shaped_str, argv[i] );
                     if ( strcmp(is_shaped_str,"yes") == 0 ) {
@@ -172,7 +232,7 @@ int main( int argc, char ** argv )
                         is_shaped = NO;
                     }
                     break;
-                case 'w':
+                case 'w': /* Set X11  window state */
                     if ( ++i >= argc ) show_usage();
                     strcpy( window_state_str, argv[i] );
                     if ( strcmp(window_state_str,"iconic") == 0 ) {
@@ -185,19 +245,26 @@ int main( int argc, char ** argv )
                             window_state = WithdrawnState;
                     }
                     break;
-                case 'g':
+                case 'g': /* Set X11 window geometry */
                     if ( ++i >= argc ) show_usage();
                     strcpy( Geometry_str, argv[i] );
                     break;
-                case 'd':
+                case 'i': /* Initial Face Set to Display */
+                    if ( ++i >= argc ) show_usage();
+                    face_set = atol(argv[i]);
+                    if ( face_set < 0 || face_set >= FACE_SETS ) {
+                       face_set = 0;
+                    }
+                    break;
+                case 'd': /* X11 -display string */
                     if ( ++i >= argc ) show_usage();
                     strcpy( Display_str, argv[i] );
                     break;
-                case 'u':
+                case 'u': /* Update Period in milliseconds */
                     if ( ++i >= argc ) show_usage();
                     update_miliseconds = atol(argv[i]);
                     break;
-                case 'm':
+                case 'm': /* Set What this app is to monitor */
                     if ( ++i >= argc ) show_usage();
                     strcpy( command.mode_str, argv[i] );
                     if ( strcmp(command.mode_str,"load") == 0 ) {
@@ -214,13 +281,12 @@ int main( int argc, char ** argv )
                         command.index = atoi(argv[i]);
                     }
                     break;
-                case 'f':
+                case 'f': /* Set floating point scale factor for 100% */
                     if ( ++i >= argc ) show_usage();
                     scale_factor = atof(argv[i]);
                     user_selected_f = YES;
                     break;
-                    
-                case 'h':
+                case 'h': /* Show Help Text*/
                     show_usage();
                     break;
             }
@@ -463,6 +529,8 @@ int main( int argc, char ** argv )
     
                     switch (report.xbutton.button) {
                         case Button1:
+                           face_set = ( face_set + 1 ) % FACE_SETS;
+                            refresh( command, scale_factor, dirrection );
                         case Button2:
                         case Button3:
                             refresh( command, scale_factor, dirrection );
@@ -549,7 +617,7 @@ void refresh( command_type command, float scale_factor, int dirrection )
     if ( Verbose )
         printf("%f%% scaled --> face %d\n", percent, level );
 
-    redraw( level , dirrection );
+    redraw( face_set, level , dirrection );
 }
 
 
@@ -562,14 +630,11 @@ void refresh( command_type command, float scale_factor, int dirrection )
  *
  ***********************************************************************/
 
-void redraw( int level, int dirrection )
+void redraw( int face_set, int level, int dirrection )
 {
     int index;
 
-    /* Face image indexed by 2d array [level][dirrection] */
-    index = 3 * level + dirrection;
-
-
+    index = (face_set * IMAGES_PER_FACE_SET) + (3 * level) + dirrection;
 
     XCopyArea(     display, 
             template.pixmap, visible.pixmap, 
@@ -585,7 +650,8 @@ void redraw( int level, int dirrection )
             0, 0,
             template.attributes.width, 
             template.attributes.height,
-            6, 5 ); 
+            face_offset[ (face_set * 2) + 0 ],
+            face_offset[ (face_set * 2) + 1 ] );
  
     flush_expose( win ); 
     XCopyArea(     display, 
@@ -652,15 +718,10 @@ void getPixmaps()
          * A value of 40000 seems reasonable for many situations requiring
          reasonable but not perfect color matches.
   */
-        
-	char ** face_name[16] = { 
-               face_0l, face_0c, face_0r,
-               face_1l, face_1c, face_1r,
-               face_2l, face_2c, face_2r,
-               face_3l, face_3c, face_3r, 
-               face_4l, face_4c, face_4r, face_god };
-
-    for (i=0; i<16; i++ ) {
+       
+    int face_array_size = FACE_SETS * IMAGES_PER_FACE_SET;
+ 
+    for (i=0; i<face_array_size; i++ ) {
         face[i].attributes.valuemask |=
                 (XpmReturnPixels | XpmReturnExtensions | XpmCloseness);
         face[i].attributes.closeness = 50001;
@@ -924,36 +985,38 @@ float get_command( command_type command, float scale_factor )
 void show_usage()
 {
 
-printf("
-%s
-usage: [-v] [-h] 
-       [-s yes/no] [-w withdrawn/iconic/normal] 
-       [-m cpu/load/uptime] 
-       [-f scale_factor ] [-u miliseconds] 
-       [-d dislay ] [-g geometry ]
-
-
--g                        geometry:  ie: 64x64+10+10
--d                             dpy:  Display. ie: 127.0.0.1:0.0
--v                                :  Verbose.
--h                                :  Help. This screen.
--w         withdrawn/iconic/normal:  Windows: Iconic, Normal, or Withdrawn
--s                          yes/no:  Shaped window: y or n
--m cpu/load/command[command index]:  Use load, cpu, or external command
--f                    scale_factor:  Scale: floating point number = 100%% bloody
--u                     miliseconds:  Update period in miliseconds
-
-Defaults:
-   -g 64x64+10+10
-   -d NULL
-   -w withdrawn
-   -s yes
-   -m load
-   -f 1.0 for CPU / -f 2.0 for LOAD
-   -u 999
-
-
-", VERSION );
+printf("\n"
+"%s\n"
+"usage: [-v] [-h] \n"
+"       [-s yes/no] [-w withdrawn/iconic/normal] \n"
+"       [-m cpu/load/uptime] \n"
+"       [-f scale_factor ] [-u miliseconds] \n"
+"       [-d dislay ] [-g geometry ]\n"
+"       [-i initial_set_of_images ]\n"
+"\n"
+"\n"
+"-g                        geometry:  ie: 64x64+10+10\n"
+"-d                             dpy:  Display. ie: 127.0.0.1:0.0\n"
+"-v                                :  Verbose.\n"
+"-h                                :  Help. This screen.\n"
+"-w         withdrawn/iconic/normal:  Windows: Iconic, Normal, or Withdrawn\n"
+"-s                          yes/no:  Shaped window: y or n\n"
+"-m cpu/load/command[command index]:  Use load, cpu, or external command\n"
+"-f                    scale_factor:  Scale: floating point number = 100%% bloody\n"
+"-u                     miliseconds:  Update period in miliseconds\n"
+"-i                        face_set:  0=first, 1=second, etc\n"
+"\n"
+"Defaults:\n"
+"   -g 64x64+10+10\n"
+"   -d NULL\n"
+"   -w withdrawn\n"
+"   -s yes\n"
+"   -m load\n"
+"   -f 1.0 for CPU / -f 2.0 for LOAD\n"
+"   -u 999\n"
+"   -i 0 ( doom faces )\n"
+"\n"
+"\n", VERSION );
 
 exit(-1);
 
